@@ -1808,12 +1808,11 @@ do_toggle_contrast_action (GsdMediaKeysManager *manager)
 
 static void
 power_action (GsdMediaKeysManager *manager,
-              const char          *action,
-              gboolean             allow_interaction)
+              const char          *action)
 {
         g_dbus_proxy_call (manager->priv->logind_proxy,
                            action,
-                           g_variant_new ("(b)", allow_interaction),
+                           g_variant_new ("(b)", TRUE),
                            G_DBUS_CALL_FLAGS_NONE,
                            G_MAXINT,
                            manager->priv->bus_cancellable,
@@ -1822,8 +1821,7 @@ power_action (GsdMediaKeysManager *manager,
 
 static void
 do_config_power_action (GsdMediaKeysManager *manager,
-                        const gchar *config_key,
-                        gboolean in_lock_screen)
+                        const gchar *config_key)
 {
         GsdPowerActionType action_type;
 
@@ -1831,17 +1829,16 @@ do_config_power_action (GsdMediaKeysManager *manager,
                                            config_key);
         switch (action_type) {
         case GSD_POWER_ACTION_SUSPEND:
-                power_action (manager, "Suspend", !in_lock_screen);
+                power_action (manager, "Suspend");
                 break;
         case GSD_POWER_ACTION_INTERACTIVE:
-                if (!in_lock_screen)
-                        gnome_session_shutdown (manager);
+                gnome_session_shutdown (manager);
                 break;
         case GSD_POWER_ACTION_SHUTDOWN:
-                power_action (manager, "PowerOff", !in_lock_screen);
+                power_action (manager, "PowerOff");
                 break;
         case GSD_POWER_ACTION_HIBERNATE:
-                power_action (manager, "Hibernate", !in_lock_screen);
+                power_action (manager, "Hibernate");
                 break;
         case GSD_POWER_ACTION_BLANK:
         case GSD_POWER_ACTION_LOGOUT:
@@ -1853,94 +1850,73 @@ do_config_power_action (GsdMediaKeysManager *manager,
 }
 
 static void
-update_screen_cb (GObject             *source_object,
-                  GAsyncResult        *res,
-                  gpointer             user_data)
+update_brightness_cb (GObject             *source_object,
+                      GAsyncResult        *res,
+                      gpointer             user_data)
 {
         GError *error = NULL;
         int percentage;
         GVariant *new_percentage;
         GsdMediaKeysManager *manager = GSD_MEDIA_KEYS_MANAGER (user_data);
+        const char *icon, *debug;
+
+        if (G_DBUS_PROXY (source_object) == manager->priv->power_keyboard_proxy) {
+                icon = "keyboard-brightness-symbolic";
+                debug = "keyboard";
+        } else {
+                icon = "display-brightness-symbolic";
+                debug = "screen";
+        }
 
         new_percentage = g_dbus_proxy_call_finish (G_DBUS_PROXY (source_object),
                                                    res, &error);
         if (new_percentage == NULL) {
-                g_warning ("Failed to set new screen percentage: %s",
-                           error->message);
+                g_warning ("Failed to set new %s percentage: %s",
+                           debug, error->message);
                 g_error_free (error);
                 return;
         }
 
         /* update the dialog with the new value */
         g_variant_get (new_percentage, "(i)", &percentage);
-        if (percentage >= 0)
-                show_osd (manager, "display-brightness-symbolic", NULL, percentage);
+        show_osd (manager, icon, NULL, percentage);
         g_variant_unref (new_percentage);
 }
 
 static void
-do_screen_brightness_action (GsdMediaKeysManager *manager,
-                             MediaKeyType type)
-{
-        if (manager->priv->connection == NULL ||
-            manager->priv->power_screen_proxy == NULL) {
-                g_warning ("No existing D-Bus connection trying to handle power keys");
-                return;
-        }
-
-        /* call into the power plugin */
-        g_dbus_proxy_call (manager->priv->power_screen_proxy,
-                           type == SCREEN_BRIGHTNESS_UP_KEY ? "StepUp" : "StepDown",
-                           NULL,
-                           G_DBUS_CALL_FLAGS_NONE,
-                           -1,
-                           NULL,
-                           update_screen_cb,
-                           manager);
-}
-
-static void
-update_keyboard_cb (GObject             *source_object,
-                    GAsyncResult        *res,
-                    gpointer             user_data)
-{
-        GError *error = NULL;
-        guint percentage;
-        GVariant *new_percentage;
-        GsdMediaKeysManager *manager = GSD_MEDIA_KEYS_MANAGER (user_data);
-
-        new_percentage = g_dbus_proxy_call_finish (G_DBUS_PROXY (source_object),
-                                                   res, &error);
-        if (new_percentage == NULL) {
-                g_warning ("Failed to set new keyboard percentage: %s",
-                           error->message);
-                g_error_free (error);
-                return;
-        }
-
-        /* update the dialog with the new value */
-        g_variant_get (new_percentage, "(i)", &percentage);
-        show_osd (manager, "keyboard-brightness-symbolic", NULL, percentage);
-        g_variant_unref (new_percentage);
-}
-
-static void
-do_keyboard_brightness_action (GsdMediaKeysManager *manager,
-                               MediaKeyType type)
+do_brightness_action (GsdMediaKeysManager *manager,
+                      MediaKeyType type)
 {
         const char *cmd;
+        GDBusProxy *proxy;
+
+        switch (type) {
+        case KEYBOARD_BRIGHTNESS_UP_KEY:
+        case KEYBOARD_BRIGHTNESS_DOWN_KEY:
+        case KEYBOARD_BRIGHTNESS_TOGGLE_KEY:
+                proxy = manager->priv->power_keyboard_proxy;
+                break;
+        case SCREEN_BRIGHTNESS_UP_KEY:
+        case SCREEN_BRIGHTNESS_DOWN_KEY:
+                proxy = manager->priv->power_screen_proxy;
+                break;
+        default:
+                g_assert_not_reached ();
+        }
 
         if (manager->priv->connection == NULL ||
-            manager->priv->power_keyboard_proxy == NULL) {
+            proxy == NULL) {
                 g_warning ("No existing D-Bus connection trying to handle power keys");
                 return;
         }
 
         switch (type) {
         case KEYBOARD_BRIGHTNESS_UP_KEY:
+        case SCREEN_BRIGHTNESS_UP_KEY:
                 cmd = "StepUp";
                 break;
         case KEYBOARD_BRIGHTNESS_DOWN_KEY:
+        case SCREEN_BRIGHTNESS_DOWN_KEY:
                 cmd = "StepDown";
                 break;
         case KEYBOARD_BRIGHTNESS_TOGGLE_KEY:
@@ -1951,13 +1927,13 @@ do_keyboard_brightness_action (GsdMediaKeysManager *manager,
         }
 
         /* call into the power plugin */
-        g_dbus_proxy_call (manager->priv->power_keyboard_proxy,
+        g_dbus_proxy_call (proxy,
                            cmd,
                            NULL,
                            G_DBUS_CALL_FLAGS_NONE,
                            -1,
                            NULL,
-                           update_keyboard_cb,
+                           update_brightness_cb,
                            manager);
 }
 
@@ -2178,37 +2154,23 @@ do_action (GsdMediaKeysManager *manager,
 		do_toggle_contrast_action (manager);
 		break;
         case POWER_KEY:
-                do_config_power_action (manager, "button-power", FALSE);
+                do_config_power_action (manager, "button-power");
                 break;
         case SLEEP_KEY:
-                do_config_power_action (manager, "button-sleep", FALSE);
+                do_config_power_action (manager, "button-sleep");
                 break;
         case SUSPEND_KEY:
-                do_config_power_action (manager, "button-suspend", FALSE);
+                do_config_power_action (manager, "button-suspend");
                 break;
         case HIBERNATE_KEY:
-                do_config_power_action (manager, "button-hibernate", FALSE);
-                break;
-        case POWER_KEY_NO_DIALOG:
-                do_config_power_action (manager, "button-power", TRUE);
-                break;
-        case SLEEP_KEY_NO_DIALOG:
-                do_config_power_action (manager, "button-sleep", TRUE);
-                break;
-        case SUSPEND_KEY_NO_DIALOG:
-                do_config_power_action (manager, "button-suspend", TRUE);
-                break;
-        case HIBERNATE_KEY_NO_DIALOG:
-                do_config_power_action (manager, "button-hibernate", TRUE);
+                do_config_power_action (manager, "button-hibernate");
                 break;
         case SCREEN_BRIGHTNESS_UP_KEY:
         case SCREEN_BRIGHTNESS_DOWN_KEY:
-                do_screen_brightness_action (manager, type);
-                break;
         case KEYBOARD_BRIGHTNESS_UP_KEY:
         case KEYBOARD_BRIGHTNESS_DOWN_KEY:
         case KEYBOARD_BRIGHTNESS_TOGGLE_KEY:
-                do_keyboard_brightness_action (manager, type);
+                do_brightness_action (manager, type);
                 break;
         case BATTERY_KEY:
                 do_battery_action (manager);
