@@ -224,7 +224,7 @@ start_gnome_software_with_mode (const gchar *mode)
 
         argv = g_new0 (gchar *, 3);
         argv[0] = g_build_filename (BINDIR, "gnome-software", NULL);
-        argv[1] = g_strdup_printf ("--mode %s", mode);
+        argv[1] = g_strdup_printf ("--mode=%s", mode);
         ret = g_spawn_async (NULL, argv, NULL, 0, NULL, NULL, NULL, &error);
         if (!ret) {
                 g_warning ("Failed to spawn gnome-software: %s", error->message);
@@ -1270,6 +1270,9 @@ check_offline_update_cb (gpointer user_data)
                 message = ngettext ("An important OS update has been installed.",
                                     "Important OS updates have been installed.",
                                     num_packages);
+
+                /* no need to keep the file around anymore */
+                clear_offline_updates_message ();
         } else {
                 /* get error details */
                 manager->priv->offline_update_error = pk_error_new ();
