@@ -38,23 +38,17 @@
 #include <gdk/gdkx.h>
 #include <gtk/gtk.h>
 
-#define GNOME_DESKTOP_USE_UNSTABLE_API
-
-#include <libgnome-desktop/gnome-rr-config.h>
-#include <libgnome-desktop/gnome-rr.h>
-#include <libgnome-desktop/gnome-pnp-ids.h>
-
 #include "gnome-settings-profile.h"
 #include "gsd-enums.h"
 #include "gsd-xsettings-manager.h"
 #include "gsd-xsettings-gtk.h"
 #include "xsettings-manager.h"
 #include "fontconfig-monitor.h"
+#include "gsd-remote-display-manager.h"
 
 #define GNOME_XSETTINGS_MANAGER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GNOME_TYPE_XSETTINGS_MANAGER, GnomeXSettingsManagerPrivate))
 
 #define MOUSE_SETTINGS_SCHEMA     "org.gnome.settings-daemon.peripherals.mouse"
-#define BACKGROUND_SETTINGS_SCHEMA "org.gnome.desktop.background"
 #define INTERFACE_SETTINGS_SCHEMA "org.gnome.desktop.interface"
 #define SOUND_SETTINGS_SCHEMA     "org.gnome.desktop.sound"
 #define PRIVACY_SETTINGS_SCHEMA     "org.gnome.desktop.privacy"
@@ -227,17 +221,8 @@
  */
 #define DPI_FALLBACK 96
 
-/* The minimum resolution at which we turn on a window-scale of 2 */
 #define HIDPI_LIMIT (DPI_FALLBACK * 2)
 
-/* The minimum screen height at which we turn on a window-scale of 2;
- * below this there just isn't enough vertical real estate for GNOME
- * apps to work, and it's better to just be tiny */
-#define HIDPI_MIN_HEIGHT 1200
-
-/* From http://en.wikipedia.org/wiki/4K_resolution#Resolutions_of_common_formats */
-#define SMALLEST_4K_WIDTH 3656
-
 typedef struct _TranslationEntry TranslationEntry;
 typedef void (* TranslationFunc) (GnomeXSettingsManager *manager,
                                   TranslationEntry      *trans,
@@ -270,7 +255,7 @@ struct GnomeXSettingsManagerPrivate
 
         GsdXSettingsGtk   *gtk;
 
-        GnomeRRScreen     *rr_screen;
+        GsdRemoteDisplayManager *remote_display;
 
         guint              shell_name_watch_id;
         gboolean           have_shell;
@@ -373,8 +358,6 @@ static TranslationEntry translations [] = {
         { "org.gnome.settings-daemon.peripherals.mouse", "double-click",   "Net/DoubleClickTime",  translate_int_int },
         { "org.gnome.settings-daemon.peripherals.mouse", "drag-threshold", "Net/DndDragThreshold", translate_int_int },
 
-        { "org.gnome.desktop.background", "show-desktop-icons",    "Gtk/ShellShowsDesktop",   translate_bool_int },
-
         { "org.gnome.desktop.interface", "gtk-color-palette",      "Gtk/ColorPalette",        translate_string_string },
         { "org.gnome.desktop.interface", "font-name",              "Gtk/FontName",            translate_string_string },
         { "org.gnome.desktop.interface", "gtk-key-theme",          "Gtk/KeyThemeName",        translate_string_string },
@@ -393,7 +376,6 @@ static TranslationEntry translations [] = {
         { "org.gnome.desktop.interface", "gtk-im-module",          "Gtk/IMModule",            translate_string_string },
         { "org.gnome.desktop.interface", "icon-theme",             "Net/IconThemeName",       translate_string_string },
         { "org.gnome.desktop.interface", "menubar-accel",          "Gtk/MenuBarAccel",        translate_string_string },
-        { "org.gnome.desktop.interface", "enable-animations",      "Gtk/EnableAnimations",    translate_bool_int },
         { "org.gnome.desktop.interface", "cursor-theme",           "Gtk/CursorThemeName",     translate_string_string },
         /* cursor-size is handled via the Xft side as it needs the scaling factor */
 
@@ -441,148 +423,33 @@ get_dpi_from_gsettings (GnomeXSettingsManager *manager)
         return dpi * factor;
 }
 
-static GnomeRROutput *
-get_primary_output (GnomeRRScreen *screen)
-{
-        GnomeRROutput *primary = NULL;
-        GnomeRROutput **outputs;
-        guint i;
-
-        outputs = gnome_rr_screen_list_outputs (screen);
-        if (outputs == NULL || outputs[0] == NULL)
-                return NULL;
-        for (i = 0; outputs[i] != NULL; i++) {
-                if (gnome_rr_output_get_is_primary (outputs[i])) {
-                        primary = outputs[i];
-                        break;
-                }
-        }
-        if (primary == NULL)
-                primary = outputs[0];
-
-        return primary;
-}
-
-static gboolean
-primary_monitor_is_4k (GnomeRROutput *primary)
-{
-        GnomeRRMode *mode;
-
-        mode = gnome_rr_output_get_current_mode (primary);
-        if (gnome_rr_mode_get_width (mode) >= SMALLEST_4K_WIDTH)
-                return TRUE;
-        return FALSE;
-}
-
-static gboolean
-primary_monitor_on_hdmi (GnomeRROutput *primary)
-{
-        const char *name;
-
-        name = gnome_rr_output_get_name (primary);
-        if (name == NULL ||
-            strstr (name, "HDMI") == NULL)
-                return FALSE;
-        return TRUE;
-}
-
-static gboolean
-primary_monitor_should_skip_resolution_check (GnomeRROutput *primary)
-{
-        if (!primary_monitor_is_4k (primary) && primary_monitor_on_hdmi (primary))
-                return TRUE;
-
-        return FALSE;
-}
-
-static void
-get_dimensions_xrandr (GnomeRROutput *primary,
-                       int           *width,
-                       int           *height,
-                       int           *width_mm,
-                       int           *height_mm)
-{
-        GnomeRRMode *mode;
-
-        mode = gnome_rr_output_get_current_mode (primary);
-        *width = gnome_rr_mode_get_width (mode);
-        *height = gnome_rr_mode_get_height (mode);
-
-        gnome_rr_output_get_physical_size (primary,
-                                           width_mm,
-                                           height_mm);
-}
-
-static void
-get_dimensions_gdk (int *width,
-                    int *height,
-                    int *width_mm,
-                    int *height_mm)
-{
-        GdkDisplay *display;
-        GdkScreen *screen;
-        GdkRectangle rect;
-        int primary;
-        int monitor_scale;
-
-        display = gdk_display_get_default ();
-        screen = gdk_display_get_default_screen (display);
-        primary = gdk_screen_get_primary_monitor (screen);
-        gdk_screen_get_monitor_geometry (screen, primary, &rect);
-        monitor_scale = gdk_screen_get_monitor_scale_factor (screen, primary);
-
-        *width = rect.width * monitor_scale;
-        *height = rect.height * monitor_scale;
-
-        *width_mm = gdk_screen_get_monitor_width_mm (screen, primary);
-        *height_mm = gdk_screen_get_monitor_height_mm (screen, primary);
-}
-
 static int
 get_window_scale (GnomeXSettingsManager *manager)
 {
 	GSettings  *interface_settings;
         int window_scale;
-        int width, height;
+        GdkRectangle rect;
+        GdkDisplay *display;
+        GdkScreen *screen;
         int width_mm, height_mm;
+        int monitor_scale;
         double dpi_x, dpi_y;
 
 	interface_settings = g_hash_table_lookup (manager->priv->settings, INTERFACE_SETTINGS_SCHEMA);
         window_scale =
                 g_settings_get_uint (interface_settings, SCALING_FACTOR_KEY);
         if (window_scale == 0) {
-                GnomeRROutput *output = NULL;
-
-                window_scale = 1;
-
-                if (manager->priv->rr_screen)
-                        output = get_primary_output (manager->priv->rr_screen);
-
-                if (output) {
-                        if (primary_monitor_should_skip_resolution_check (output))
-                                goto out;
-
-                        get_dimensions_xrandr (output,
-                                               &width, &height,
-                                               &width_mm, &height_mm);
-                } else {
-                        /* Before the D-Bus DisplayConfig service exported by
-                         * Mutter becomes available, use the current information
-                         * that GDK has from the X server; in simple cases, this
-                         * will hopefully keep us from switching the window_scale
-                         * during startup.
-                         */
-                        get_dimensions_gdk (&width, &height,
-                                            &width_mm, &height_mm);
-                }
-
-                if (height < HIDPI_MIN_HEIGHT)
-                        goto out;
+                display = gdk_display_get_default ();
+                screen = gdk_display_get_default_screen (display);
+                gdk_screen_get_monitor_geometry (screen, 0, &rect);
+                width_mm = gdk_screen_get_monitor_width_mm (screen, 0);
+                height_mm = gdk_screen_get_monitor_height_mm (screen, 0);
+                monitor_scale = gdk_screen_get_monitor_scale_factor (screen, 0);
 
                 window_scale = 1;
                 if (width_mm > 0 && height_mm > 0) {
-                        dpi_x = (double)width / (width_mm / 25.4);
-                        dpi_y = (double)height / (height_mm / 25.4);
+                        dpi_x = (double)rect.width * monitor_scale / (width_mm / 25.4);
+                        dpi_y = (double)rect.height * monitor_scale / (height_mm / 25.4);
                         /* We don't completely trust these values so both
                            must be high, and never pick higher ratio than
                            2 automatically */
@@ -591,7 +458,6 @@ get_window_scale (GnomeXSettingsManager *manager)
                 }
         }
 
-out:
         return window_scale;
 }
 
@@ -1078,30 +944,26 @@ start_shell_monitor (GnomeXSettingsManager *manager)
 }
 
 static void
-on_rr_screen_changed (GnomeRRScreen         *screen,
-                      GnomeXSettingsManager *manager)
+force_disable_animation_changed (GObject    *gobject,
+                                 GParamSpec *pspec,
+                                 GnomeXSettingsManager *manager)
 {
-        update_xft_settings (manager);
-        queue_notify (manager);
-}
+        gboolean force_disable, value;
+        int i;
 
-static void
-on_rr_screen_acquired (GObject      *object,
-                       GAsyncResult *result,
-                       gpointer      data)
-{
-        GnomeXSettingsManager *manager = data;
-        GnomeRRScreen *rr_screen;
+        g_object_get (gobject, "force-disable-animations", &force_disable, NULL);
+        if (force_disable)
+                value = FALSE;
+        else {
+                GSettings *settings;
 
-        rr_screen = gnome_rr_screen_new_finish (result, NULL);
-        if (!rr_screen)
-                return;
+                settings = g_hash_table_lookup (manager->priv->settings, "org.gnome.desktop.interface");
+                value = g_settings_get_boolean (settings, "enable-animations");
+        }
 
-        manager->priv->rr_screen = rr_screen;
-        g_signal_connect (rr_screen, "changed",
-                          G_CALLBACK (on_rr_screen_changed), manager);
-
-        on_rr_screen_changed (rr_screen, manager);
+        for (i = 0; manager->priv->managers [i]; i++) {
+                xsettings_manager_set_int (manager->priv->managers [i], "Gtk/EnableAnimations", value);
+        }
 }
 
 gboolean
@@ -1122,17 +984,15 @@ gnome_xsettings_manager_start (GnomeXSettingsManager *manager,
                 return FALSE;
         }
 
-        gnome_rr_screen_new_async (gdk_screen_get_default (),
-                                   on_rr_screen_acquired,
-                                   manager);
+        manager->priv->remote_display = gsd_remote_display_manager_new ();
+        g_signal_connect (G_OBJECT (manager->priv->remote_display), "notify::force-disable-animations",
+                          G_CALLBACK (force_disable_animation_changed), manager);
 
         manager->priv->settings = g_hash_table_new_full (g_str_hash, g_str_equal,
                                                          NULL, (GDestroyNotify) g_object_unref);
 
         g_hash_table_insert (manager->priv->settings,
                              MOUSE_SETTINGS_SCHEMA, g_settings_new (MOUSE_SETTINGS_SCHEMA));
-        g_hash_table_insert (manager->priv->settings,
-                             BACKGROUND_SETTINGS_SCHEMA, g_settings_new (BACKGROUND_SETTINGS_SCHEMA));
         g_hash_table_insert (manager->priv->settings,
                              INTERFACE_SETTINGS_SCHEMA, g_settings_new (INTERFACE_SETTINGS_SCHEMA));
         g_hash_table_insert (manager->priv->settings,
@@ -1177,6 +1037,9 @@ gnome_xsettings_manager_start (GnomeXSettingsManager *manager,
                           G_CALLBACK (gtk_modules_callback), manager);
         gtk_modules_callback (manager->priv->gtk, NULL, manager);
 
+        /* Animation settings */
+        force_disable_animation_changed (G_OBJECT (manager->priv->remote_display), NULL, manager);
+
         /* Xft settings */
         update_xft_settings (manager);
 
@@ -1210,15 +1073,7 @@ gnome_xsettings_manager_stop (GnomeXSettingsManager *manager)
 
         g_debug ("Stopping xsettings manager");
 
-        if (manager->priv->rr_screen != NULL) {
-                g_signal_handlers_disconnect_by_func (manager->priv->rr_screen,
-                                                      (gpointer) on_rr_screen_changed,
-                                                      manager);
-                g_clear_object (&manager->priv->rr_screen);
-        }
-
-        if (p->shell_name_watch_id > 0)
-                g_bus_unwatch_name (p->shell_name_watch_id);
+        g_clear_object (&manager->priv->remote_display);
 
         if (p->managers != NULL) {
                 for (i = 0; p->managers [i]; ++i)
@@ -1235,6 +1090,9 @@ gnome_xsettings_manager_stop (GnomeXSettingsManager *manager)
 
         stop_fontconfig_monitor (manager);
 
+        if (manager->priv->shell_name_watch_id > 0)
+                g_bus_unwatch_name (manager->priv->shell_name_watch_id);
+
         if (p->settings != NULL) {
                 g_hash_table_destroy (p->settings);
                 p->settings = NULL;
